/*
 * "Copyright (c) 2011 The Regents of the Signet Group, University of Padova.
 * All rights reserved."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE SIGNET GROUP BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE SIGNET GROUP
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE SIGNET GROUP SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
 * AND THE SIGNET GROUP HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/*
 * 	header file for 6LoWPAN protocol
 *
 * 	6lowpan module general constants and structures are defined 
 * 	starting point to define all header files to run 6lowpan
 * 
 *
 * @author Giulio Ministeri <minister@dei.unipd.it>
 */
 

#ifndef LOWPAN_H
#define LOWPAN_H
						
typedef struct	{		// struct init compression
	memory_id_t packet;
	ieee154_saddr_t srcAddr;
	ieee154_saddr_t dstAddr;
#ifdef SLP_CONTEXT
	slp_context_t cont;
#endif
} __attribute__((packed))slp_initcp_t ;	


typedef struct	{		// struct init decompression
	ieee154_saddr_t srcAddr;
	ieee154_saddr_t dstAddr;
#ifdef SLP_CONTEXT
	slp_context_t cont;
#endif
} __attribute__((packed))slp_initdp_t ;	


typedef struct	{		// struct internal in 6lowpan send
	memory_id_t packet;
#ifdef SLP_MESH
	slp_mesh_opt_t mesh;
#endif
	slp_frag_opt_t frag;
	uint16_t byteWritten;
	bool valid;
} __attribute__((packed))slp_snd_entry_t ;


typedef struct	{		// struct internal in 6lowpan receive
	memory_id_t packet;
#ifdef SLP_MESH
	slp_mesh_opt_t* mesh;
#endif
	slp_frag_opt_t frag;
	bool checksumElide;
	uint16_t byteWritten;
} __attribute__((packed))slp_rec_entry_t ;


typedef uint8_t slp_cp_error_t;
typedef uint8_t slp_dp_error_t;

enum slp_compress {
	SLP_CP_SUCCESS = 0,
	SLP_CP_FAIL = 1,
	SLP_CP_PKT_DONE = 2
};

enum slp_decompress	{
	SLP_DP_SUCCESS = 0,
	SLP_DP_FAIL = 1,
	SLP_DP_MEM_FAIL = 2,
	SLP_DP_REC_FAIL = 3,
	SLP_DP_FWD = 4
};


enum	{
	NALP_MAX_VALUE = 64,
	ELIDE_CHECKSUM_UDP = 0,
	
	SLP_PCK_DEFAULT_VALUE = 0xAA,
};



const uint8_t SLP_EUI64_SHORT_ADDR[]   =     { 0x00, 0x00, 0x00, 0xFF, 0xFE, 0x00};



#endif
