/*
 * "Copyright (c) 2011 The Regents of the Signet Group, University of Padova.
 * All rights reserved."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE SIGNET GROUP BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE SIGNET GROUP
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE SIGNET GROUP SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
 * AND THE SIGNET GROUP HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/*
 * 		RFC4944 header file 
 *
 * 	functions to handle fragmentation and mesh header
 * 	and also to place payload datas in buffers
 * 
 *
 * @author Giulio Ministeri <minister@dei.unipd.it>
 */


/******************* prima parte: costruzione e compressione **********************************/
#ifdef SLP_MESH
void setMeshFields (slp_mesh_opt_t* meshHeader, uint8_t Hops, ieee154_saddr_t originAddress, ieee154_saddr_t finalDestAddress)	{
	meshHeader->pattern = SLP_MESH_PATTERN;
	meshHeader->v = 1;
	meshHeader->f = 1;
	meshHeader->hopsLeft = Hops;
	meshHeader->originator = originAddress;
	meshHeader->finalDest = finalDestAddress;
}
#endif




void setFragFields (slp_frag_opt_t* fragHeader, uint16_t size, nx_uint16_t tag)	{
#ifndef TEST_SEND
	fragHeader->pattern = SLP_FRAG_FIRST_PATTERN;
	fragHeader->size = size;
	fragHeader->tag = tag;
	fragHeader->offset = 0;	
#endif
}



uint16_t fillMsg (slp_mesh_opt_t* meshHeader, slp_frag_opt_t* fragHeader, void* L2Payload, uint16_t L2PayloadLength, void* L3Payload, uint16_t L3PayloadLength, void* compHeader, uint8_t compSize, uint8_t originSize, uint16_t* bytered)	{
#ifndef TEST_SEND
	uint8_t bytewritten = 0;
	uint16_t temp, temp2;
	uint8_t tt;
	
		// writing mesh header	
#ifdef SLP_MESH	
	if (meshHeader)	{			
		memcpy(L2Payload, meshHeader), sizeof(slp_mesh_opt_t));
		L2Payload += sizeof (slp_mesh_opt_t);
		L2PayloadLength -= sizeof (slp_mesh_opt_t);
		bytewritten += sizeof (slp_mesh_opt_t);
	}
#endif

		// writing fragmentation header (if necessary)
	if (fragHeader->offset || L3PayloadLength - originSize + compSize > L2PayloadLength)	{
		if (fragHeader->offset == 0)
			tt = sizeof(slp_frag_opt_t) - sizeof(fragHeader->offset);
		else
			tt = sizeof(slp_frag_opt_t);
		
		memcpy(L2Payload, fragHeader, tt);
		L2Payload += tt;
		L2PayloadLength -= tt;
		bytewritten += tt;
		fragHeader->pattern = SLP_FRAG_OTHER_PATTERN;
	}
		
	// writing 6lp header (if present)
	if (fragHeader->offset == 0)	{
		memcpy(L2Payload, compHeader, compSize);
		L2Payload += compSize;
		L2PayloadLength -= compSize;
		L3Payload += originSize;
		L3PayloadLength -= originSize;
		bytewritten += compSize;
		*bytered = originSize;
		
		// preparing for payload writing
		temp2 = originSize;
	}
	else	{
		// preparing for payload writing		
		temp = (fragHeader->offset)<<3;
		L3Payload += temp;
		L3PayloadLength -= temp;
		temp2 = 0;
	}	
		// writing the payload
	if (L3PayloadLength <= L2PayloadLength)	{
		memcpy(L2Payload, L3Payload, L3PayloadLength);
		bytewritten += L3PayloadLength;
		*bytered += L3PayloadLength;
	}
	else	{
		L2PayloadLength += temp2;	
		L2PayloadLength >>= 3;
		temp = (L2PayloadLength<<3) - temp2;
		memcpy (L2Payload, L3Payload, temp);
		bytewritten += temp;
		*bytered += temp;
		fragHeader->offset += L2PayloadLength;
	}	
	return bytewritten;
#else
	return 0;
#endif

}




/******************* seconda parte: lettura e decompressione **********************************/

uint8_t readOptHeader (void* L2Payload, slp_mesh_opt_t** meshHeader, slp_frag_opt_t* fragHeader)	{
#ifndef TEST_RECEIVE
	uint8_t decodedByte = 0;
	slp_frag_opt_t* tempFrag;	
#ifdef SLP_MESH
	*meshHeader = (slp_mesh_opt_t*) L2Payload;	
	
	if ((*meshHeader)->pattern == SLP_MESH_PATTERN)	{
		L2Payload += sizeof(slp_mesh_opt_t);
 		decodedByte += sizeof(slp_mesh_opt_t);
	}	
	else
		*meshHeader = NULL;
#endif	
	tempFrag = (slp_frag_opt_t*)(L2Payload);

	if (tempFrag->pattern == SLP_FRAG_FIRST_PATTERN)	{
		memcpy(fragHeader, L2Payload, sizeof(slp_frag_opt_t) - sizeof(fragHeader->offset));
		decodedByte += sizeof(slp_frag_opt_t) - sizeof(fragHeader->offset);
	}
	else if (tempFrag->pattern == SLP_FRAG_OTHER_PATTERN)	{
		memcpy(fragHeader, L2Payload, sizeof(slp_frag_opt_t));
		decodedByte += sizeof(slp_frag_opt_t);
	}	
	return decodedByte;
#else
	return 0;
#endif
}




#ifdef SLP_MESH
void decrementMeshHops(void* L2Payload, slp_packet_entry_t* entry)	{
	if (entry->mesh.hopsleft == 0 && entry->mesh.originator == NULL && entry->mesh.finalDest == NULL)	{
		return;			// altro return da controllare
	
	entry->mesh.hops--;
	memcpy(L2Payload, &(entry->mesh), sizeof(entry->mesh));		
}
#endif




uint16_t fillPayload (slp_frag_opt_t* fragHeader, void* L3Payload, void* L2Payload, uint16_t L2PayloadLength)	{
#ifndef TEST_RECEIVE	
	uint16_t start;
	uint16_t i, t;
	t=0;
	start = fragHeader->offset<<3;
	L3Payload += start;
	for (i=0; i < L2PayloadLength; i++)	{
		if (*(uint8_t*)(L3Payload + i) == SLP_PCK_DEFAULT_VALUE)
			t++;
	}
	
	memcpy(L3Payload, L2Payload, L2PayloadLength);
	return t;	
#else
	return 0;
#endif
}
