/*
 * "Copyright (c) 2011 The Regents of the Signet Group, University of Padova.
 * All rights reserved."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE SIGNET GROUP BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE SIGNET GROUP
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE SIGNET GROUP SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
 * AND THE SIGNET GROUP HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/*
 *	compression header file 
 *
 * 	compression and decompression functions are defined
 * 
 * 
 *
 * @author Giulio Ministeri <minister@dei.unipd.it>
 */


/************************** Compression side ******************************/
// TODO: No context based compression, neither for compression or decompression, 
//	 no traffic Class and flow labels handling
//	 NHCDispatch handle only UDP header, no extension header


// unicast address matching search, returns the size of written byte 
// WARNING: no context based compression !!!
uint8_t IPHCAddressCompress (slp_ip6_addr_t* addr, void* pointer, ieee154_saddr_t* macAddr)	{
	void* temp;
	uint8_t byteWritten;
	
	temp = addr;
	
// trying to match address

	if (memcmp(temp, &SLP_LINKUNI_NET_ADDR, sizeof(SLP_LINKUNI_NET_ADDR))!=0)	{
//no source address match
		memcpy(pointer, temp, sizeof(slp_ip6_addr_t));
		return sizeof(slp_ip6_addr_t) ;
	}
	else	{
		temp += sizeof(SLP_LINKUNI_NET_ADDR);
		if (memcmp(temp, &SLP_EUI64_SHORT_ADDR, sizeof(SLP_EUI64_SHORT_ADDR))!=0)	{
//source address match FE80::XXXX:XXXX:XXXX:XXXX
			memcpy(pointer, temp, sizeof(slp_ip6_addr_t) - sizeof(SLP_LINKUNI_NET_ADDR));
			return sizeof(slp_ip6_addr_t) - sizeof(SLP_LINKUNI_NET_ADDR);
		}
		else	{
			temp += sizeof(SLP_EUI64_SHORT_ADDR);			
			if (memcmp(temp, macAddr, sizeof (ieee154_saddr_t))==0 )
//source address match FE80::0000:00FF:FE00:SRC_MAC_ADDRESS
				return 0;
			else	{
//source address match FE80::0000:00FF:FE00:XXXX
				memcpy(pointer, temp, sizeof(ieee154_saddr_t));
				return sizeof(ieee154_saddr_t);
			}
		}
	}
	return byteWritten;
}

uint8_t IPHCCompress (void* L3Payload, void* buffer, uint8_t* originSize, ieee154_saddr_t* srcAddr, ieee154_saddr_t* destAddr, bool checksumElide)	{
#if !defined TEST_SEND && !defined TEST_HCCOMPRESS
	void* temp;
	uint8_t n;
	slp_ip6_header_t* IPv6header = NULL;
	slp_UDP_header_t* UDPheader = NULL;
	slp_iphc_header_t* IPHCDispatch = NULL;
	uint8_t headerSize = 0;
	slp_nhc_header_t* NHCDispatch = NULL;
	*originSize = 0;
	
	IPv6header = (slp_ip6_header_t*) L3Payload;
	L3Payload += sizeof(slp_ip6_header_t);
	*originSize += sizeof(slp_ip6_header_t);
	
	IPHCDispatch = (slp_iphc_header_t*) buffer;
	buffer += sizeof(slp_iphc_header_t);
	headerSize += sizeof(slp_iphc_header_t);
	
//writing basic flags	
	IPHCDispatch->pattern = SLP_IPHC_PATTERN;
	IPHCDispatch->cid = SLP_IPHC_CID_NOPE;
	IPHCDispatch->tf = SLP_IPHC_TF_NOPE;
		
//writing NHC dispatch	
	if ( IPv6header->nextHeader == UDP_PROTOCOL_NUM)	{
		UDPheader = (slp_UDP_header_t*) L3Payload ;
		L3Payload += sizeof(slp_UDP_header_t);
		*originSize += sizeof(slp_UDP_header_t);
		
		IPHCDispatch->nh = SLP_IPHC_NH_NHC;

		NHCDispatch = (slp_nhc_header_t*) buffer;
		buffer += sizeof(slp_nhc_header_t);
		headerSize += sizeof(slp_nhc_header_t);
		
		NHCDispatch->pattern = SLP_NHC_UDP_PATTERN;
	 }	

	 else	{
	 	IPHCDispatch->nh = SLP_IPHC_NH_INLINE;
	 	memcpy(buffer, &(IPv6header->nextHeader), sizeof(uint8_t));
	 	buffer += sizeof(uint8_t);
	 	headerSize += sizeof(uint8_t);
	 }

//writing hop limit field		
	switch (IPv6header->hopLimit)	{
	case 1:
		IPHCDispatch->hlim = SLP_IPHC_HLIM_1;
			break;
	case 64:
		IPHCDispatch->hlim = SLP_IPHC_HLIM_64;
			break;
	case 255:
		IPHCDispatch->hlim = SLP_IPHC_HLIM_255;
			break;
	default	:	
		IPHCDispatch->hlim = SLP_IPHC_HLIM_INLINE;
		memcpy (buffer, &(IPv6header->hopLimit), sizeof(uint8_t));
		buffer += sizeof(uint8_t);
		headerSize += sizeof(uint8_t);
			break;
	}

	IPHCDispatch->sac = SLP_IPHC_SAC_STATELESS;
	
// trying to match source address
	n = IPHCAddressCompress (&IPv6header->source, buffer, srcAddr);
	buffer += n;
	headerSize += n;
	switch (n)	{
	case 16:
		IPHCDispatch->sam = SLP_IPHC_SAM_128;
			break;
	case 8:
		IPHCDispatch->sam = SLP_IPHC_SAM_64;
			break;
	case 2:
		IPHCDispatch->sam = SLP_IPHC_SAM_16;
			break;
	case 0:		
		IPHCDispatch->sam = SLP_IPHC_SAM_0;
			break;
	default:
		return 0;
	}
		
	IPHCDispatch->dac = SLP_IPHC_DAC_STATELESS;
	
// trying to match destination address
	temp = &(IPv6header->destination);
//first i check the multicast address compression
	if (memcmp(temp, &SLP_IPHC_MULTICAST_PREFIX, sizeof(SLP_IPHC_MULTICAST_PREFIX)) == 0)	{	
//it's a multicast address
		IPHCDispatch->m = SLP_IPHC_M_YES;
		n = sizeof(SLP_IPHC_MULTICAST_PREFIX) + SLP_IPHC_MULTICAST_SCOPE_DIM;
// how many zeros are there?
		while (*((uint8_t*)(temp+n)) == 0)
			n++;

 		if (n>=14 && memcmp (temp, &SLP_LINKMUL_NET_ADDR, sizeof(SLP_LINKMUL_NET_ADDR)) == 0 )	{
//multicast address match FF02::XX
 			IPHCDispatch->dam = SLP_IPHC_DAM_0_8;
			temp +=15;
			memcpy(buffer, temp, sizeof(uint8_t));		// writing the last byte
			buffer += sizeof(uint8_t);
			headerSize += sizeof(uint8_t);
		}
		else if (n>=11)	{
 			memcpy (buffer, temp + sizeof(SLP_IPHC_MULTICAST_PREFIX), sizeof(SLP_IPHC_MULTICAST_SCOPE_DIM));
 			buffer += SLP_IPHC_MULTICAST_SCOPE_DIM;
			if (n>=13)	{
//multicast address match FFXX::00XX:XXXX
				IPHCDispatch->dam = SLP_IPHC_DAM_16_32;
 				temp += 13;
 				memcpy(buffer, temp, SLP_IPHC_MULTICAST_32_DIM - SLP_IPHC_MULTICAST_SCOPE_DIM);
 				buffer += SLP_IPHC_MULTICAST_32_DIM - SLP_IPHC_MULTICAST_SCOPE_DIM;
 				headerSize += SLP_IPHC_MULTICAST_32_DIM;
			}
			else 	{	
//multicast address match FFXX::00XX:XXXX:XXXX
				IPHCDispatch->dam = SLP_IPHC_DAM_64_48;
        	                temp += 11;
                	        memcpy(buffer, temp, sizeof(SLP_IPHC_MULTICAST_48_DIM) - SLP_IPHC_MULTICAST_SCOPE_DIM);
	                  	buffer += sizeof(SLP_IPHC_MULTICAST_48_DIM) - SLP_IPHC_MULTICAST_SCOPE_DIM;
	                       	headerSize += sizeof(SLP_IPHC_MULTICAST_48_DIM);
			}
		}
		else	{		
//no multicast address match
			IPHCDispatch->dam = SLP_IPHC_DAM_128;
                        memcpy (buffer, &IPv6header->destination, sizeof(slp_ip6_addr_t));
                        buffer += sizeof(slp_ip6_addr_t);
                        headerSize += sizeof(slp_ip6_addr_t);
		}
	}
	else	{
		IPHCDispatch->m = SLP_IPHC_M_NOPE;
// unicast address
		n = IPHCAddressCompress (&IPv6header->destination, buffer, destAddr);
		buffer += n;
		headerSize += n;
		switch (n)	{
		case 16:
			IPHCDispatch->dam = SLP_IPHC_DAM_128;
				break;
		case 8:
			IPHCDispatch->dam = SLP_IPHC_DAM_64_48;
				break;
		case 2:
			IPHCDispatch->dam = SLP_IPHC_DAM_16_32;
				break;
		case 0:		
			IPHCDispatch->dam = SLP_IPHC_DAM_0_8;
				break;
		default: 
			return 0;
		}
	}

// UDP compression... if it is
	if (IPHCDispatch->nh == SLP_IPHC_NH_NHC && NHCDispatch->pattern == SLP_NHC_UDP_PATTERN)	{
			
	 	if ((UDPheader->sourcePort & SLP_NHC_UDP_PORT_COMPRESS_12BIT_MASK) == SLP_NHC_UDP_PORT_COMPRESS_12BIT && (UDPheader->destinationPort & SLP_NHC_UDP_PORT_COMPRESS_12BIT_MASK) == SLP_NHC_UDP_PORT_COMPRESS_12BIT)	{
	 		NHCDispatch->p = SLP_NHC_P_12BIT_COMPRESS;
	 		n = (UDPheader->sourcePort & ~SLP_NHC_UDP_PORT_COMPRESS_12BIT_MASK) << 4;
	 		n += (UDPheader->destinationPort & ~SLP_NHC_UDP_PORT_COMPRESS_12BIT_MASK);

	 		memcpy(buffer, &n, sizeof(uint8_t));
	 		buffer += sizeof(uint8_t);
	 		headerSize += sizeof(uint8_t);
	 	}	
	 	else if ((UDPheader->sourcePort & SLP_NHC_UDP_PORT_COMPRESS_8BIT_MASK) == SLP_NHC_UDP_PORT_COMPRESS_8BIT)	{
	 		NHCDispatch->p = SLP_NHC_P_8BIT_SRC_COMPRESS;
	 		n = UDPheader->sourcePort & ~SLP_NHC_UDP_PORT_COMPRESS_8BIT_MASK;
	 		memcpy(buffer, &n, sizeof(uint8_t));
	 		buffer += sizeof(uint8_t);
	 		headerSize += sizeof(uint8_t);
	 		memcpy (buffer, &UDPheader->destinationPort, sizeof(nx_uint16_t));
	 		buffer += sizeof(nx_uint16_t);
	 		headerSize += sizeof(nx_uint16_t);
	 	}
	 	else if ((UDPheader->destinationPort & SLP_NHC_UDP_PORT_COMPRESS_8BIT_MASK) == SLP_NHC_UDP_PORT_COMPRESS_8BIT)	{
	 		NHCDispatch->p = SLP_NHC_P_8BIT_DST_COMPRESS;
	 		memcpy (buffer, &UDPheader->sourcePort, sizeof(nx_uint16_t));
	 		buffer += sizeof(nx_uint16_t);
	 		headerSize += sizeof(nx_uint16_t);
	 		n = UDPheader->destinationPort & ~SLP_NHC_UDP_PORT_COMPRESS_8BIT_MASK;
	 		memcpy(buffer, &n, sizeof(uint8_t));
	 		buffer += sizeof(uint8_t);
	 		headerSize += sizeof(uint8_t);
	 	}
	 	else	{
	 		NHCDispatch->p = SLP_NHC_P_INLINE;
	 		memcpy(buffer, &UDPheader->sourcePort, sizeof(nx_uint16_t));
	 		buffer += sizeof(nx_uint16_t);
	 		headerSize += sizeof(nx_uint16_t);
	 		memcpy(buffer, &UDPheader->destinationPort, sizeof(nx_uint16_t));
	 		buffer += sizeof(nx_uint16_t);
	 		headerSize += sizeof(nx_uint16_t);
	 	}
	 	
	 	if (checksumElide)
			NHCDispatch->c = SLP_NHC_CKSUM_COMPRESS;
			
		 else	{
			NHCDispatch->c = SLP_NHC_CKSUM_INLINE;
			memcpy(buffer, &UDPheader->checksum, sizeof(nx_uint16_t));
	 		buffer += sizeof(nx_uint16_t);
	 		headerSize += sizeof(nx_uint16_t);
	 	}
	 }

	return headerSize;
#else
	return 0;
#endif
	}




// ************************* decompression side ************************//

uint8_t IPHCAddressDecompress (uint8_t code, slp_ip6_addr_t* addr, void* pointer, ieee154_saddr_t* macAddr)	{
	void* temp;
	uint8_t byteRead;
	
	memset(addr, 0, sizeof(slp_ip6_addr_t));	
	temp = addr;
	
//using source address mode code since for unicast address is the same as for destination address
	switch ( code )	{
	case SLP_IPHC_SAM_128:
//no unicast address match
		memcpy(addr, pointer, sizeof(slp_ip6_addr_t));
		byteRead = sizeof(slp_ip6_addr_t);
			break;
			
	case SLP_IPHC_SAM_64:		
//unicast address match FE80::XXXX:XXXX:XXXX:XXXX
		memcpy(temp, &SLP_LINKUNI_NET_ADDR , sizeof(SLP_LINKUNI_NET_ADDR));
		temp += sizeof(SLP_LINKUNI_NET_ADDR);
		memcpy(temp, pointer, sizeof(slp_ip6_addr_t) - sizeof(SLP_LINKUNI_NET_ADDR));
		byteRead = sizeof(slp_ip6_addr_t) - sizeof(SLP_LINKUNI_NET_ADDR);
			break;
	
	case SLP_IPHC_SAM_16:		
//unicast address match FE80::00FF:FE00:XXXX
		memcpy(temp, &SLP_LINKUNI_NET_ADDR, sizeof(SLP_LINKUNI_NET_ADDR));
		temp += sizeof(SLP_LINKUNI_NET_ADDR);
		memcpy(temp, &SLP_EUI64_SHORT_ADDR, sizeof(SLP_EUI64_SHORT_ADDR));
		temp += sizeof(SLP_EUI64_SHORT_ADDR);
		memcpy (temp, pointer, sizeof(ieee154_saddr_t));
		byteRead = sizeof(ieee154_saddr_t);
			break;				
	
	case SLP_IPHC_SAM_0:
//unicast address match FE80::00FF:FE00:macAddr
		memcpy(temp, &SLP_LINKUNI_NET_ADDR, sizeof(SLP_LINKUNI_NET_ADDR));
		temp += sizeof(SLP_LINKUNI_NET_ADDR);
		memcpy(temp, &SLP_EUI64_SHORT_ADDR, sizeof(SLP_EUI64_SHORT_ADDR));
		temp += sizeof(SLP_EUI64_SHORT_ADDR);
		memcpy(temp , macAddr, sizeof(ieee154_saddr_t));
		byteRead = 0;
			break;	
	default:
		byteRead = 0;	
	}
	return byteRead;
}


uint8_t IPHCDecompress (void* L2Payload, uint16_t L2PayloadLength, void* L3Payload, ieee154_saddr_t* macSrcAddr, ieee154_saddr_t* macDestAddr, uint8_t* originSize, slp_context_t* context, bool* checksumElide)	{
#if !defined TEST_HCDECOMPRESS	&& !TEST_RECEIVE
	uint8_t decodedByte = 0;
	slp_ip6_header_t* IPv6header = NULL;
	slp_UDP_header_t* UDPheader = NULL;
	slp_iphc_header_t* IPHCDispatch = NULL;
	slp_nhc_header_t* NHCDispatch = NULL;
	void* temp;
	uint8_t n;
	
	*originSize = 0;
	IPHCDispatch = (slp_iphc_header_t*) L2Payload;
	L2Payload += sizeof(slp_iphc_header_t);
	*originSize += sizeof(slp_iphc_header_t);
	
	IPv6header = (slp_ip6_header_t*) L3Payload;
	L3Payload += sizeof(slp_ip6_header_t);
	decodedByte += sizeof(slp_ip6_header_t);
	
	if (IPHCDispatch->pattern != SLP_IPHC_PATTERN)	{
		*originSize = 0;
		return 0;
	}
	
	IPv6header->version = 6;
	IPv6header->payloadLength = 0;
	
//CID field: 			//first field in IPHC header after IPHC Dispatch
	if (IPHCDispatch->cid == SLP_IPHC_CID_EXT)	{
	#ifdef SLP_CONTEXT
		memcpy(context, L2Payload, sizeof(slp_context_t));
		L2Payload += sizeof(slp_context_t);
		*originSize += sizeof(slp_context_t);
	#else
		L2Payload += 1;					//WARNING little trick
		*originSize += 1;
	#endif
	}
	
//TF field:				//no operations
	switch (IPHCDispatch->tf)	{
	
	case SLP_IPHC_TF_ECN_DSCP_FW:
		L2Payload += 4;		// just to take note...
		*originSize += 4;	
			break;
			
	case SLP_IPHC_TF_ECN_FW:
		L2Payload += 3;
		*originSize += 3;
			break;
			
	case SLP_IPHC_TF_ECN_DSCP:
		L2Payload += 1;
		*originSize += 1;
			break;
			
	case SLP_IPHC_TF_NOPE:
		break;
	}
		
//NH field:

	if (IPHCDispatch->nh == SLP_IPHC_NH_NHC)	{
		NHCDispatch = (slp_nhc_header_t*) L2Payload;
		L2Payload += sizeof(slp_nhc_header_t);
		*originSize += sizeof(slp_nhc_header_t);
	}
	else	{
		memcpy(&(IPv6header->nextHeader), L2Payload, sizeof(uint8_t));
		L2Payload += sizeof(uint8_t);
		*originSize += sizeof(uint8_t);
	}
	
//HLIM field:		
	switch (IPHCDispatch->hlim)	{
	
	case SLP_IPHC_HLIM_1:
		IPv6header->hopLimit = 1;
			break;
	
	case SLP_IPHC_HLIM_64:
		IPv6header->hopLimit = 64;
			break;

	case SLP_IPHC_HLIM_255:
		IPv6header->hopLimit = 255;
			break;
	
	case SLP_IPHC_HLIM_INLINE:
		memcpy(&(IPv6header->hopLimit), L2Payload, sizeof(uint8_t));
		L2Payload += sizeof(uint8_t);
		*originSize += sizeof(uint8_t);
			break;
	}

//SAC field:
// NO CONTEXT 
		
//SAM field:

	n = IPHCAddressDecompress ((uint8_t) IPHCDispatch->sam, &IPv6header->source, L2Payload,  macSrcAddr);
	L2Payload += n;
	*originSize += n;
		
//DAC field: 	


//DAM field:
	if (IPHCDispatch->m == SLP_IPHC_M_NOPE)	{
		n = IPHCAddressDecompress((uint8_t) IPHCDispatch->dam, &IPv6header->destination, L2Payload,  macDestAddr);
		L2Payload += n;
		*originSize += n;
	}	
	else	{
		memset(&(IPv6header->destination), 0, sizeof(slp_ip6_addr_t));
		temp = &(IPv6header->destination);
		switch ( IPHCDispatch->dam )	{
		case SLP_IPHC_DAM_128:
			memcpy(&(IPv6header->destination), L2Payload, sizeof(slp_ip6_addr_t));
			L2Payload += sizeof(slp_ip6_addr_t);
			*originSize += sizeof(slp_ip6_addr_t);
				break;
		case SLP_IPHC_DAM_64_48:
			memcpy (temp, &SLP_IPHC_MULTICAST_PREFIX, sizeof(SLP_IPHC_MULTICAST_PREFIX));
			temp += sizeof(SLP_IPHC_MULTICAST_PREFIX);
			memcpy(temp, L2Payload, SLP_IPHC_MULTICAST_SCOPE_DIM);
			temp = &(IPv6header->destination);
			temp += sizeof(slp_ip6_addr_t) - (SLP_IPHC_MULTICAST_48_DIM - SLP_IPHC_MULTICAST_SCOPE_DIM);
			memcpy(temp, L2Payload + SLP_IPHC_MULTICAST_SCOPE_DIM, SLP_IPHC_MULTICAST_48_DIM - SLP_IPHC_MULTICAST_SCOPE_DIM);
			L2Payload += SLP_IPHC_MULTICAST_48_DIM;
			*originSize += SLP_IPHC_MULTICAST_48_DIM;
				break;
		case SLP_IPHC_DAM_16_32:
			memcpy (temp, &SLP_IPHC_MULTICAST_PREFIX, sizeof(SLP_IPHC_MULTICAST_PREFIX));
			temp += sizeof(SLP_IPHC_MULTICAST_PREFIX);
			memcpy(temp, L2Payload, SLP_IPHC_MULTICAST_SCOPE_DIM);
			temp = &(IPv6header->destination);
			temp += sizeof(slp_ip6_addr_t) - (SLP_IPHC_MULTICAST_32_DIM - SLP_IPHC_MULTICAST_SCOPE_DIM) ;
			memcpy(temp, L2Payload + SLP_IPHC_MULTICAST_SCOPE_DIM, SLP_IPHC_MULTICAST_32_DIM - SLP_IPHC_MULTICAST_SCOPE_DIM);
			L2Payload += SLP_IPHC_MULTICAST_32_DIM;
			*originSize += SLP_IPHC_MULTICAST_32_DIM;	
				break;	
		case SLP_IPHC_DAM_0_8:
			memcpy (temp, &SLP_IPHC_MULTICAST_8_ADDR, sizeof(SLP_IPHC_MULTICAST_8_ADDR));
			temp += sizeof(slp_ip6_addr_t) - SLP_IPHC_MULTICAST_8_DIM;
			memcpy(temp, L2Payload, SLP_IPHC_MULTICAST_8_DIM);
			L2Payload += SLP_IPHC_MULTICAST_8_DIM;
			*originSize += SLP_IPHC_MULTICAST_8_DIM;
				break;		
		}
	}
		
//NHCDispatch decompression:	
	if (NHCDispatch)	{
		if (NHCDispatch->pattern == SLP_NHC_UDP_PATTERN)	{	
			IPv6header->nextHeader = UDP_PROTOCOL_NUM;		
			UDPheader = (slp_UDP_header_t*) L3Payload;
			L3Payload += sizeof(slp_UDP_header_t);
			decodedByte += sizeof(slp_UDP_header_t);
			IPv6header->payloadLength += SLP_UDP_HEADER_LENGTH;
		
			switch (NHCDispatch->p)	{
	
			case SLP_NHC_P_12BIT_COMPRESS:
				UDPheader->sourcePort = SLP_NHC_UDP_PORT_COMPRESS_12BIT + ((*(uint8_t*)(L2Payload) & 0xF0)>>4);
				UDPheader->destinationPort = SLP_NHC_UDP_PORT_COMPRESS_12BIT + (*(uint8_t*)(L2Payload) & 0x0F);		
				L2Payload += sizeof(uint8_t);
				*originSize += sizeof(uint8_t);
					break;
	
			case SLP_NHC_P_8BIT_SRC_COMPRESS:
				UDPheader->sourcePort = SLP_NHC_UDP_PORT_COMPRESS_8BIT + *(uint8_t*)(L2Payload);
				L2Payload += sizeof(uint8_t);
				*originSize += sizeof(uint8_t);
				memcpy(&(UDPheader->destinationPort), L2Payload, sizeof(nx_uint16_t));
				L2Payload += sizeof(nx_uint16_t);
				*originSize += sizeof(nx_uint16_t);
					break;
			
			case SLP_NHC_P_8BIT_DST_COMPRESS:
				memcpy(&(UDPheader->sourcePort), L2Payload, sizeof(nx_uint16_t));
				L2Payload += sizeof(nx_uint16_t);
				*originSize += sizeof(nx_uint16_t);
				UDPheader->destinationPort = (SLP_NHC_UDP_PORT_COMPRESS_8BIT) + *(uint8_t*)(L2Payload);
				L2Payload += sizeof(uint8_t);
				*originSize += sizeof(uint8_t);
					break;

			default:
				memcpy(&(UDPheader->sourcePort), L2Payload, sizeof(nx_uint16_t));
				L2Payload += sizeof(nx_uint16_t);
				*originSize += sizeof(nx_uint16_t);
				memcpy(&(UDPheader->destinationPort), L2Payload, sizeof(nx_uint16_t));
				L2Payload += sizeof(nx_uint16_t);
				*originSize += sizeof(nx_uint16_t);
					break;
			}
			
			if (NHCDispatch->c == SLP_NHC_CKSUM_COMPRESS)
				*checksumElide = TRUE;
			else	{
				memcpy(&(UDPheader->checksum), L2Payload, sizeof(nx_uint16_t));
				L2Payload += sizeof(nx_uint16_t);
				*originSize += sizeof(nx_uint16_t);
			}
		}
		
		else	{
			// WARNING!!!!!!!! Extension header!!!!
		}
	}

	return decodedByte;
#else
	return 0;
#endif
	
}
