/*
 * "Copyright (c) 2011 The Regents of the Signet Group, University of Padova.
 * All rights reserved."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE SIGNET GROUP BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE SIGNET GROUP
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE SIGNET GROUP SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
 * AND THE SIGNET GROUP HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/*
 * 		header file for compression and decompression functions
 *
 * 		header files with structures and constants for compression
 * 		and decompression functions properly defined in rfc6282
 * 
 *
 * @author Giulio Ministeri <minister@dei.unipd.it>
 */


typedef nx_struct	{
	nx_uint8_t sci:4;
	nx_uint8_t dci:4;
} __attribute__((packed))slp_context_t ;


/*
	 0   1   2   3   4   5   6   7   8   9   0   1   2   3   4   5
     +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
     | 0 | 1 | 1 |  TF   |NH | HLIM  |CID|SAC|  SAM  | M |DAC|  DAM  |
     +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+


                    LOWPAN_IPHC base Encoding

*/

typedef nx_struct	{
	nx_uint16_t pattern:3;
	nx_uint16_t tf:2;
	nx_uint16_t nh:1;
	nx_uint16_t hlim:2;
	nx_uint16_t cid:1;
	nx_uint16_t sac:1;
	nx_uint16_t sam:2;
	nx_uint16_t m:1;
	nx_uint16_t dac:1;
	nx_uint16_t dam:2;
} __attribute__((packed))slp_iphc_header_t;


/*
			 0   1   2   3   4   5   6   7
                     +---+---+---+---+---+---+---+---+
                     | 1 | 1 | 1 | 1 | 0 | C |   P   |
                     +---+---+---+---+---+---+---+---+

                      Figure 14: UDP Header Encoding

*/


typedef nx_struct	{
	nx_uint8_t pattern:5;
	nx_uint8_t c:1;
	nx_uint8_t p:2;
} __attribute__((packed))slp_nhc_header_t;	



enum RFC6282_header_values	{
	
	SLP_IPHC_PATTERN = 3,

	SLP_IPHC_TF_ECN_DSCP_FW = 0,
	SLP_IPHC_TF_ECN_FW = 1,
	SLP_IPHC_TF_ECN_DSCP = 2,
	SLP_IPHC_TF_NOPE = 3,
	
	SLP_IPHC_NH_INLINE = 0,
	SLP_IPHC_NH_NHC = 1,
	
	SLP_IPHC_HLIM_INLINE = 0,
	SLP_IPHC_HLIM_1 = 1,
	SLP_IPHC_HLIM_64 = 2,
	SLP_IPHC_HLIM_255 = 3,
	
	SLP_IPHC_CID_NOPE = 0,
	SLP_IPHC_CID_EXT = 1,
	
	SLP_IPHC_SAC_STATELESS = 0,
	SLP_IPHC_SAC_STATEFUL = 1,
	
	SLP_IPHC_SAM_128 = 0,
	SLP_IPHC_SAM_64 = 1,
	SLP_IPHC_SAM_16 = 2,
	SLP_IPHC_SAM_0 = 3,
	
	SLP_IPHC_M_NOPE = 0,
	SLP_IPHC_M_YES = 1,
	
	SLP_IPHC_DAC_STATELESS = 0,
	SLP_IPHC_DAC_STATEFUL = 1,

	SLP_IPHC_DAM_128 = 0,
	SLP_IPHC_DAM_64_48 = 1,
	SLP_IPHC_DAM_16_32 = 2,
	SLP_IPHC_DAM_0_8 = 3,
	SLP_IPHC_MULTICAST_48_DIM = 6,
	SLP_IPHC_MULTICAST_32_DIM = 4,
	SLP_IPHC_MULTICAST_8_DIM = 1,
	SLP_IPHC_MULTICAST_SCOPE_DIM = 1	
	
};

enum RFC6282_udp_nhc_values	{


	SLP_NHC_UDP_PATTERN = 0x1E,
	SLP_NHC_UDP_PORT_COMPRESS_8BIT = 0xF000,
	SLP_NHC_UDP_PORT_COMPRESS_8BIT_MASK = 0xFF00,
	SLP_NHC_UDP_PORT_COMPRESS_12BIT = 0xF0B0,
	SLP_NHC_UDP_PORT_COMPRESS_12BIT_MASK = 0xFFF0,
	
	SLP_NHC_P_12BIT_COMPRESS = 3,
	SLP_NHC_P_8BIT_SRC_COMPRESS = 2,
	SLP_NHC_P_8BIT_DST_COMPRESS = 1,
	SLP_NHC_P_INLINE = 0,
	SLP_NHC_CKSUM_COMPRESS = 1,
	SLP_NHC_CKSUM_INLINE = 0
};

enum RFC6282_MISC	{
	IPHC_MAX_SIZE = 50
};


const uint8_t SLP_IPHC_MULTICAST_8_ADDR[] = {0xFF, 0x02};
const uint8_t SLP_IPHC_MULTICAST_PREFIX = 0xFF;

