/*
 * "Copyright (c) 2011 The Regents of the Signet Group, University of Padova.
 * All rights reserved."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE SIGNET GROUP BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE SIGNET GROUP
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE SIGNET GROUP SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
 * AND THE SIGNET GROUP HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/*		Header file for ICMPv6 protocol
 * 
 *	
 * 		constants and structure of ICMPv6 protocol
 * 
 * 
 *
 * @author Giulio Ministeri <minister@dei.unipd.it>
 */


#ifndef ICMPv6_H
#define ICMPv6_H

typedef nx_struct	{
	nx_uint8_t type;
	nx_uint8_t code;
	nx_uint16_t checksum;
} slp_icmpv6_mes_t;




typedef nx_struct	{
	nx_uint16_t identifier;
	nx_uint16_t sequenceNumber;
} slp_icmpv6_echo_t;


typedef nx_struct	{
	nx_uint8_t curHopLimit;
	nx_uint8_t m: 1;
	nx_uint8_t o: 1;
	nx_uint8_t reserved: 6 ;
	nx_uint16_t routerLifetime;
	nx_uint32_t reachableTime;
	nx_uint32_t retransTime;
} slp_icmpv6_rout_adv_t;


enum ICMPv6Const	{
	ICMPv6_PROTOCOL_NUM = 58
};


enum informationalMessages	{

	ICMP_ECHOREQUEST = 128,
	ICMP_ECHOREPLY = 129,
	SLP_ROUTER_SOLIC_TYPE = 133,
	SLP_ROUTER_ADV_TYPE = 134,
	SLP_NEIGH_SOLIC_TYPE = 135,
	SLP_NEIGH_ADV_TYPE = 136,
	SLP_MULT_ROUTER_SOLIC_TYPE = 152,
	SLP_MULT_ROUTER_ADV_TYPE = 151,
	SLP_MULT_ROUTER_TERM_TYPE = 153,
	
	
	SLP_ECHO_REQUEST_CODE = 0	
};
	
	

enum errorMessages	{
	SLP_DEST_UNREACHABLE = 1,
	SLP_PCK_TOO_BIG = 2,
	SLP_TIME_EXCEED = 3,
	SLP_PARAMETER_PROBL = 4
};


#endif
