/*
 * "Copyright (c) 2011 The Regents of the Signet Group, University of Padova.
 * All rights reserved."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE SIGNET GROUP BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE SIGNET GROUP
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE SIGNET GROUP SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
 * AND THE SIGNET GROUP HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/*  IPv6.c
 *  functions to handle IPv6 addresses
 *  functions to compare addresses or prefixes and to find if equal
 *  
 * 
 * 
 *
 * @author Giulio Ministeri <minister@dei.unipd.it>
 */


#ifndef IPv6_C
#define IPv6_C

bool isMyNetwork ( slp_ip6_addr_t* addr, slp_ip6_addr_t myAddresses[])	{
  	uint8_t n;
	for (n = 0; n < SLP_IPV6_MAX_ADDR; n++)	{
		if (memcmp(NET_ADDR(addr), NET_ADDR(&myAddresses[n]), sizeof(slp_ip6_net_addr_t)) == 0)
			return TRUE;
	}
	return FALSE;
}

bool isMyAddress ( slp_ip6_addr_t* addr, slp_ip6_addr_t myAddresses[])	{
  	uint8_t n;		
	for (n = 0; n < SLP_IPV6_MAX_ADDR; n++)	{
		
		if (memcmp(addr, &myAddresses[n], sizeof(slp_ip6_addr_t)) == 0)
			return TRUE;
	}
	return FALSE;
}
  
  
bool isLinkLocal (slp_ip6_addr_t* addr)	{
	return( (memcmp( addr, &SLP_LINKUNI_NET_ADDR, sizeof (SLP_LINKUNI_NET_ADDR)) == 0) || (memcmp( addr, &SLP_LINKMUL_NET_ADDR, sizeof (SLP_LINKMUL_NET_ADDR)) == 0));
}


uint8_t recognizeAddr (slp_ip6_addr_t* address)	{
	uint8_t temp[2];
	void* tt = address;
	memcpy(temp, tt, 2);
	temp[1]= temp[1] & 0x0f;
	
	if(memcmp(&temp, &SLP_LINKMUL_NET_ADDR, sizeof(SLP_LINKMUL_NET_ADDR)) ==0)
		return SLP_ADDR_LINK_MUL;
	
	if(memcmp(tt, &SLP_SITEMUL_NET_ADDR, sizeof (SLP_SITEMUL_NET_ADDR)) == 0)
		return SLP_ADDR_SITE_MUL;
		
	if(memcmp(tt, &SLP_GLOBMUL_NET_ADDR, sizeof (SLP_GLOBMUL_NET_ADDR)) == 0)
		return SLP_ADDR_GLOB_MUL;
		
	if(memcmp(tt, &SLP_LINKUNI_NET_ADDR, sizeof (SLP_LINKUNI_NET_ADDR)) == 0)
		return SLP_ADDR_LINK_UNI;
	
	if(memcmp(tt, &SLP_SITEUNI_NET_ADDR, sizeof (SLP_SITEUNI_NET_ADDR)) == 0)
		return SLP_ADDR_SITE_UNI;
		
	return SLP_ADDR_GLOB_UNI;
		
}

#endif
