/*
 * "Copyright (c) 2011 The Regents of the Signet Group, University of Padova.
 * All rights reserved."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE SIGNET GROUP BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE SIGNET GROUP
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE SIGNET GROUP SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
 * AND THE SIGNET GROUP HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/*
 * IPv6.h header file
 *
 * constants and structure referred to IPv6 standard
 * 
 * definition of IPv6 header structure and IPv6 address structure
 *
 * @author Giulio Ministeri <minister@dei.unipd.it>
 */


#ifndef IPv6_H
#define IPv6_H


#define NET_ADDR(x) ((slp_ip6_net_addr_t*)x)

typedef nx_struct {
	nx_uint8_t addr[16];	
} slp_ip6_addr_t;


typedef nx_struct	{
	nx_uint8_t addr[8];
} slp_ip6_net_addr_t;


typedef nx_struct {
	nx_uint8_t addr[8];
} slp_EUI64_addr_t;

typedef struct	{
	slp_ip6_addr_t addr;
	uint8_t interfaceID;
	uint8_t addrType;
} slp_ip6_entry_t;
	

typedef nx_struct	{
	nx_uint32_t version:4;
	nx_uint32_t trafficClass:8;
	nx_uint32_t flowLabel:20;
	nx_uint16_t payloadLength;
	nx_uint8_t nextHeader;
	nx_uint8_t hopLimit;
	slp_ip6_addr_t source;
	slp_ip6_addr_t destination;
} slp_ip6_header_t;


typedef uint8_t re_error_t;


enum routingEngineError	{
	RE_PKT_FWD = 0,
	RE_PKT_4ME = 1,
	RE_PKT_DRP = 2,
};


const uint8_t SLP_LINKUNI_NET_ADDR[] = { 0xFE, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
const uint8_t SLP_SITEUNI_NET_ADDR[] = { 0xFE, 0xC0, 0x00, 0x00, 0x00, 0x00};
const uint8_t SLP_LINKMUL_NET_ADDR[] = { 0xFF, 0x02, 0x00, 0x00, 0x00, 0x00};
const uint8_t SLP_SITEMUL_NET_ADDR[] = { 0xFF, 0x0E};
const uint8_t SLP_GLOBMUL_NET_ADDR[] = { 0xFF, 0x05};

const uint8_t SLP_NULL_ADDR[]     = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
const uint8_t SLP_LOOPBACK_ADDR[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1};


enum IFstate {
	IF_OFF = 0,
	IF_READY = 1,
};

enum IPv6Const	{
	SLP_IP6_HEADER_LENGTH = 40,
	SLP_IP6_DEFAULT_HOPS_LEFT = 255,
};

#define INTERFACE "interfaces"
enum IPv6Interfaces	{
	SLP_IF_154 = 0,
#ifdef PPP
	SLP_IF_PPP = 1,
	SLP_IF_TOT = 2,
#else
	SLP_IF_TOT = 1,
#endif
	SLP_IF_LOCAL = 255
};

enum IPv6AddressType {
	SLP_ADDR_ANY = 0,
	SLP_ADDR_LINK_UNI = 1,
	SLP_ADDR_GLOB_UNI = 2,
	SLP_ADDR_SITE_UNI = 3,
	SLP_ADDR_LINK_MUL = 4,
	SLP_ADDR_GLOB_MUL = 5,
	SLP_ADDR_SITE_MUL = 6,
	SLP_ADDR_NULL 	  = 7,
	SLP_ADDR_LOOPBACK = 8
};
#endif
