/*
 * "Copyright (c) 2011 The Regents of the Signet Group, University of Padova.
 * All rights reserved."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE SIGNET GROUP BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE SIGNET GROUP
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE SIGNET GROUP SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
 * AND THE SIGNET GROUP HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/*  Simple function to calculate checksum
 * 
 *
 * taken by RFC 1071
 * 
 * 
 *
 * @author Giulio Minister <minister@dei.unipd.it>
 */

#ifndef CHECKSUM
#define CHECKSUM

typedef nx_struct	{
	slp_ip6_addr_t source;
	slp_ip6_addr_t destination;
	nx_uint32_t upperPayloadLength;
	nx_uint16_t padd2;
	nx_uint8_t padd3;
	nx_uint8_t nextHeader;
} slp_pseudoHeader_t;


typedef struct	{
	void* pointer;
	uint16_t length;
} slp_checksum_block_t;


uint16_t checksum (slp_checksum_block_t blocks[], uint8_t elemNum)	{	//copiata e adattata da RFC1071
  	
  	uint8_t i;
  	void* addr;
  	uint16_t count;
  	
  	uint16_t cksum;
  	register uint32_t sum = 0;
        
        for (i=0; i< elemNum; i++)	{
        	addr = blocks[i].pointer;
        	count = blocks[i].length;
        
		while( count > 1 )  {
/*			 sum += get_16t(addr);*/
			 sum += *((nx_uint16_t*)(addr));
			 addr = addr + 2;
			 count -= 2;
	       }
	       if( count > 0 )
		       sum += *(uint8_t*)addr;		       
	}
	       
       while (sum>>16)
           sum = (sum & 0xffff) + (sum >> 16);
       cksum = (uint16_t)(~sum);
       return cksum;
  }
  
#endif
