/*									tab:4
 * Copyright (c) 2005 The Regents of the University  of California.  
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the
 *   distribution.
 * - Neither the name of the copyright holders nor the names of
 *   its contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * Java-side application for testing serial port communication.
 * 
 *
 * @author Phil Levis <pal@cs.berkeley.edu>
 * @date August 12 2005
 */

import java.io.IOException;

import net.tinyos.message.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;
import java.lang.*;
import java.util.Arrays;

public class SerialRoute implements MessageListener {

  private MoteIF moteIF;
  
  public SerialRoute(MoteIF moteIF) {
    this.moteIF = moteIF;
    this.moteIF.registerListener(new serialRouteMsg(), this);
  }

  public void sendPacket(serialRouteMsg payload) {
    int counter = 0;
 
    try {
	System.out.println("Sending packet ");
	moteIF.send(0, payload);
	try {Thread.sleep(1000);}
	catch (InterruptedException exception) {}
    }
    catch (IOException exception) {
      System.err.println("Exception thrown when sending packets. Exiting.");
      System.err.println(exception);
    }
  }

  public void messageReceived(int to, Message message) {
    serialRouteMsg msg = (serialRouteMsg)message;
 //   System.out.println("Received packet sequence number " + msg.get_counter());
  }
  
  private static void usage() {
    System.err.println("usage: TestSerial [-comm <source>]");
  }

private static short[] inet_pton6(String addr) {
  int c = 0;
  int end = 0;
  char[] cur = new char[5];
  char last=0;
  short[] ip6addr = new short[16];
  for (int i=0; i< ip6addr.length; i++)
	ip6addr[i]=0;

  // first fill in from the front
 for (int i=0; i< addr.length(); i+=5)	{
	for (int k=0; k<5;k++)
		cur[k]=0;

	for (int h = i; h-i<5 && h<addr.length(); h++)
		cur[h-i] = addr.charAt(h);

//	System.out.println(Arrays.toString(cur) + ", last" + last);	
	if (cur[0] == ':')	{
		if (cur[1] == ':' || last == ':')	{
			end = 1;
			break;
		}
		i -= 4;
		c += 2;
	}
	else if (cur[1] == ':')	{
		c += 1;
		ip6addr[c] = Short.parseShort(String.valueOf(cur[0]),16);
		c += 1;
		i -= 3;
		last = cur[1];
	}
	else if (cur[2] == ':')	{
		c += 1;
		ip6addr[c] =(short) (Short.parseShort(String.valueOf(cur[0]),16)*16 + Short.parseShort(String.valueOf(cur[1]),16));
		c += 1;
		i -= 2;
		last = cur[2];
	}
	else if (cur[3] == ':')	{
		ip6addr[c] = Short.parseShort(String.valueOf(cur[0]),16);
		c += 1;
		ip6addr[c] =(short) (Short.parseShort(String.valueOf(cur[1]),16)*16 + Short.parseShort(String.valueOf(cur[2]),16));
		c += 1;
		i -= 1;
		last = cur[3];
	}
	else	{
		ip6addr[c] =(short) (Short.parseShort(String.valueOf(cur[0]),16)*16 + Short.parseShort(String.valueOf(cur[1]),16));
		c += 1;
                ip6addr[c] =(short) (Short.parseShort(String.valueOf(cur[2]),16)*16 + Short.parseShort(String.valueOf(cur[3]),16));
                c += 1;
		last = cur[4];
	}
  }
  if (end == 1)	{
	last = 0;
	c = ip6addr.length - 1;
	for (int i= addr.length()-1; i>= 0; i-=5)  {
     	   for (int k=0; k<5;k++)
      	   	cur[k]=0;

	   for (int h=i; h>=0 && h-i+4>=0 ; h--)
   	        cur[h-i+4] = addr.charAt(h);

 //          System.out.println("end"+ Arrays.toString(cur) + ", last" + last);     
	   if (cur[4] == ':')	{
		if (cur[4]==':' || last == ':')
			break;
	        i += 4;
		last = cur[4];
	   }
	   else if (cur[3] == ':')	{
		ip6addr[c] = Short.parseShort(String.valueOf(cur[4]),16);
		c -= 2;
	  	last = cur[3];
		i += 3;
	   }
	   else if (cur[2] == ':')	{
		ip6addr[c] =(short) (Short.parseShort(String.valueOf(cur[3]),16)*16 + Short.parseShort(String.valueOf(cur[4]),16));
		c -= 2;
		last = cur[2];
		i += 2;
	   }
	   else if (cur[1] == ':')	{
		ip6addr[c] =(short) (Short.parseShort(String.valueOf(cur[3]),16)*16 + Short.parseShort(String.valueOf(cur[4]),16));
                c -= 1;
		ip6addr[c] = Short.parseShort(String.valueOf(cur[2]),16);
		c -= 1;
		last = cur[1];
		i += 1;
	   }
	   else	{
		ip6addr[c] =(short) (Short.parseShort(String.valueOf(cur[3]),16)*16 + Short.parseShort(String.valueOf(cur[4]),16));
                c -= 1;
		ip6addr[c] =(short) (Short.parseShort(String.valueOf(cur[1]),16)*16 + Short.parseShort(String.valueOf(cur[2]),16));
                c -= 1;
		last = cur[0];
	   }

	}
  }
  System.out.println (Arrays.toString(ip6addr));
  return ip6addr;
}


  
  public static void main(String[] args) throws Exception {
     String source = null;
    

    if (!args[0].equals("-comm")) {
	usage();
	System.exit(1);
      }
    source = args[1]; 
    PhoenixSource phoenix;
    
    if (source == null) {
      phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);
    }
    else {
      phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
    }

    MoteIF mif = new MoteIF(phoenix);
    SerialRoute serial = new SerialRoute(mif);
    serialRouteMsg payload = new serialRouteMsg();
  
   short[] destPrefix = inet_pton6(args[2]);
   payload.set_destPrefix(destPrefix);

   payload.set_prefixLength(Short.parseShort(args[3]));

   short[] nextHop = inet_pton6(args[4]);
   payload.set_nextHop(nextHop);
   
   if (args[5].equals("154"))
	payload.set_interfaceID((short)(0));
   else if (args[5].equals("PPP"))
        payload.set_interfaceID((short)(1));
   else
	System.exit(1);

    serial.sendPacket(payload);
    System.exit(0);
  }

}
