/*
 * "Copyright (c) 2011 The Regents of the Signet Group, University of Padova.
 * All rights reserved."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE SIGNET GROUP BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE SIGNET GROUP
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE SIGNET GROUP SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
 * AND THE SIGNET GROUP HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/*
 *
 * @author Giulio Ministeri <minister@dei.unipd.it>
 */


#ifndef ROUTING_H
#define ROUTING_H

/* Routing Table Entry */
typedef struct {
	slp_ip6_addr_t dest_prefix;		/* IPv6 or Prefix address of destination */			
	slp_ip6_addr_t next_hop;		/* Link local address of next_hop */
	uint8_t prefix_len;				/* Number of valid bits */		
	uint8_t interfaceID;			/* Interface's identifier */
	uint8_t metric;					/* Value metric associated */
} __attribute__((packed)) rt_entry_t;



#ifdef PRINTF

#define TO_CHAR(X) (((X) < 10) ? ('0' + (X)) : ('a' + ((X) - 10)))

struct in6_addr
  {
    union
      {
	uint8_t	u6_addr8[16];
	uint16_t u6_addr16[8];
	uint32_t u6_addr32[4];
      } in6_u;
#define s6_addr			in6_u.u6_addr8
#define s6_addr16		in6_u.u6_addr16
#define s6_addr32		in6_u.u6_addr32
  };



/** PRINT ROUTING TABLE**/

void printf_slp_ip6_addr(slp_ip6_addr_t *a) {
	uint8_t i;
	char X[16];
	memcpy(X, a, 16);
	for(i=0; i<16; i+=2) {
		printf("%x%x:", X[i],X[i+1]);
	}
	printfflush();
}

void printf_buf(char *buf, int len) {
	int i;
	for (i = 0; i < len; i++) {
	printf("%02hhx ", buf[i]);
	}
	printf("\n");
}

int inet_ntop6(struct in6_addr *addr, char *buf, int cnt) {
	uint16_t block;
	char *end = buf + cnt;
	int i, j, compressed = 0;

	for (j = 0; j < 8; j++) {
	if (buf > end - 8)
	  goto done;

	block = ntohs(addr->s6_addr16[j]);
	for (i = 4; i <= 16; i+=4) {
	  if (block > (0xffff >> i) || (compressed == 2 && i == 16)) {
		*buf++ = TO_CHAR((block >> (16 - i)) & 0xf);
	  }
	}
	if (addr->s6_addr16[j] == 0 && compressed == 0) {
	  *buf++ = ':';
	  compressed++;
	}
	if (addr->s6_addr16[j] != 0 && compressed == 1) compressed++;

	if (j < 7 && compressed != 1) *buf++ = ':';
	}
	if (compressed == 1)
	*buf++ = ':';
	done:
	*buf++ = '\0';
	return buf - (end - cnt);
}

/* printf an internet address */
void printf_in6addr(slp_ip6_addr_t *a) {
	struct in6_addr tmp;  
	static char print_buf[64];
	memcpy(&tmp, a, 16);	
	inet_ntop6(&tmp, print_buf, 64);
	printf(print_buf);
}
#endif

#endif

